#include <eopcc/cloud_io.hpp>
#include <memory>

using namespace eopcc;

PointCloud MergeByTimestamps(const PointCloud& in0, const PointCloud& in1) {
  PointCloud out;
  out.resize(in0.size() + in1.size());

  int in0_idx = 0;
  int in1_idx = 0;

  for (int i=0; i < out.size(); ++i) {
    if (in0_idx < in0.size() && in1_idx < in1.size()) {
      if (in0.gps_time[in0_idx] < in1.gps_time[in1_idx]) {
        out.set_from(i, in0, in0_idx);
        in0_idx++;
      } else {
        out.set_from(i, in1, in1_idx);
        in1_idx++;
      }
    } else if (in0_idx < in0.size()) {
      out.set_from(i, in0, in0_idx);
      in0_idx++;
    } else if (in1_idx < in1.size()) {
      out.set_from(i, in1, in1_idx);
      in1_idx++;
    }
  }

  return out;
}


int main(int argc, char* argv[]) {
  if (argc != 4) {
    std::cout << "Usage: " << argv[0] << " <input 1> <input 2> <output>" << std::endl;
    return -1;
  }

  std::unique_ptr<PointCloudReader> reader1(eopcc::CreateReader(argv[1]));
  std::unique_ptr<PointCloudReader> reader2(eopcc::CreateReader(argv[2]));
    PointCloud seed = reader1->GetSeedCloud();

    int sub_cloud_size = 50000;
    int count = 0;

    std::unique_ptr<PointCloudWriter> writer(new PLYWriter(std::string(argv[3]), seed, true));

    while (reader1->NumRemainingPoints() || reader2->NumRemainingPoints()) {
      std::cout << "Read chunk " << count << " " << reader1->NumRemainingPoints() << " " << reader2->NumRemainingPoints() << std::endl;
      count++;
      PointCloud in1 = reader1->ReadSubCloud(sub_cloud_size);
      PointCloud in2 = reader2->ReadSubCloud(sub_cloud_size);

      PointCloud out = MergeByTimestamps(in1, in2);

      writer->WriteSubCloud(out);
    }
}
